use std::net::{SocketAddr, ToSocketAddrs};

#[derive(Debug)]
pub struct Config {
    pub mappings: Vec<(String, Vec<SocketAddr>, Vec<(String, String)>)>,
}

impl Config {
    pub fn parse(str: &str) -> Result<Self, String> {
        let mut mappings = vec![];
        for (i, line) in str.lines().enumerate() {
            let i = i + 1;
            if let Some(ln) = line.strip_prefix("from ") {
                if let Some(addrs) = ln.trim().to_socket_addrs().ok() {
                    mappings.push((ln.trim().to_owned(), addrs.collect(), vec![]));
                } else {
                    return Err(format!("Line {i}: {line}: expected `from <addr:port>`"));
                }
            } else if let Some(ln) = line.strip_prefix("to ") {
                if let Some((name, addr)) = ln.split_once(char::is_whitespace) {
                    if let Some((_, _, mapping_to)) = mappings.last_mut() {
                        mapping_to.push((name.to_owned(), addr.to_owned()));
                    } else {
                        return Err(format!("Line {i}: {line}: `to ...` lines are only valid after the first `from ...` line"));
                    }
                } else {
                    return Err(format!("Line {i}: {line}: expected `to <name> <addr:port>`"));
                }
            } else if !(line.trim().is_empty() || line.trim().starts_with('#') || line.trim().starts_with("//")) {
                return Err(format!("Line {i}: expected `from` or `to`"));
            }
        }
        Ok(Self {
            mappings,
        })
    }
}
