use crate::config::Config;

use std::collections::VecDeque;
use std::{collections::HashMap, net::SocketAddr};
use std::sync::Arc;
use smol::channel::{Receiver, Sender};
use smol::lock::Mutex;
use smol::Task;
use smol::{net::{TcpListener, TcpStream}, spawn};

pub async fn run(config: Config) {
    let connections = Arc::new(Mutex::new(HashMap::new()));
    let mut tasks = vec![];
    for (mapping_from_name, mapping_from, mapping_to) in config.mappings.into_iter() {
        eprintln!(
            "[OK] listening on {mapping_from_name}:\n  from  |  {}\n    to{}",
            mapping_from.iter().enumerate().map(|(i, addr)| format!("{}{addr}", if i == 0 { "" } else { " / "})).collect::<String>(),
            mapping_to.iter().map(|(name, addr)| format!("  |  {name} {addr}")).collect::<String>(),
        );
        let mapping_to = Arc::new(mapping_to);
        for mapping_from in mapping_from {
            match TcpListener::bind(mapping_from).await {
                Ok(listener) => {
                    tasks.push(spawn(listen(mapping_from, listener, Arc::clone(&mapping_to), Arc::clone(&connections))));
                },
                Err(e) => eprintln!("[SKIP] Couldn't listen on {mapping_from}: {e}"),
            }
        }
    }
    for task in tasks {
        task.await;
    }
}

async fn listen(addr: SocketAddr, listener: TcpListener, mapping: Arc<Vec<(String, String)>>, connections: Arc<Mutex<HashMap<u64, (Task<()>, Arc<Mutex<(VecDeque<u8>, Sender<Result<bool, VecDeque<u8>>>, Receiver<Result<bool, VecDeque<u8>>>, TcpStream)>>)>>>) {
    let mut counter = u64::MAX;
    loop {
        counter = counter.wrapping_add(1);
        match listener.accept().await {
            Ok((con, _)) => {
                let mapping = mapping.clone();
                let connections = Arc::clone(&connections);
                spawn(async move {
                    if let Err(e) = crate::protocol::server(con, counter, mapping, connections).await {
                        eprintln!("[{counter}] Connection setup failed: {e}");
                    }
                }).detach();
            }
            Err(e) => {
                eprintln!("[ERR] Can't accept connections on {addr}: {e}");
                return;
            }
        }
    }
}
